# -*- coding: utf-8 -*-

import pkgutil
from io import StringIO
from operator import itemgetter

import jieba
import numpy as np
from sklearn.feature_extraction.text import CountVectorizer

from .words_sim import SimCilin

data = pkgutil.get_data(__package__, 'cilin_words.txt')
jieba.load_userdict(StringIO(data.decode()))

ci_lin = SimCilin()


def segmentation(sentence_list):
    """
    Segment a set of sentences before calculate similarity
    :param sentence_list:
    :return:
    """
    result = list()
    for s in sentence_list:
        temp_seg = jieba.cut(s)
        result.append([x for x in temp_seg])

    return result


def get_similarity(s1, s2):
    """
    Calculate the similarity of two words by Cilin
    :param s1: a word list, the result of segmentation
    :param s2:
    :return:
    """
    if len(s1) == 0 or len(s2) == 0:
        return 0.0

    chinese_list1 = []
    chinese_list2 = []
    chinese_str1 = ''
    chinese_str2 = ''
    non_chinese_str1 = ''
    non_chinese_str2 = ''
    for word in s1:
        if is_contains_chinese(word):
            chinese_list1.append(word)
            chinese_str1 += word
        else:
            non_chinese_str1 += word
    for word in s2:
        if is_contains_chinese(word):
            chinese_list2.append(word)
            chinese_str2 += word
        else:
            non_chinese_str2 += word

    sim_matrix = np.zeros((len(chinese_list1), len(chinese_list2)), dtype=float)
    for i in range(len(chinese_list1)):
        for j in range(len(chinese_list2)):
            sim_matrix[i, j] = ci_lin.compute_word_sim(chinese_list1[i], chinese_list2[j])
    chinese_sim = 0.0
    if sim_matrix.any():
        chinese_sim = (np.max(sim_matrix, axis=0).mean() + np.max(sim_matrix, axis=1).mean()) / 2

    non_chinese_sim = jaccard_similarity(non_chinese_str1, non_chinese_str2)

    chinese_len = len(chinese_str1 + chinese_str2)
    non_chinese_len = len(non_chinese_str1 + non_chinese_str2)

    return (chinese_sim * chinese_len + non_chinese_sim * non_chinese_len) / (chinese_len + non_chinese_len)


def most_similar_items(src_s, sentences, n=3):
    """
    Return the sentences most similar to the target
    :param src_s: target sentence
    :param sentences:
    :param n: return number
    :return:
    """
    src_s = [x for x in jieba.cut(src_s)]
    sentences = segmentation(sentences)
    temp = list()
    for item in sentences:
        sim_value = get_similarity(src_s, item)
        temp.append({
            'key': merge(item),
            'value': sim_value,
        })
    result = sorted(temp, key=itemgetter('value'), reverse=True)
    return result[:n]


def is_contains_chinese(s):
    """
    Tell if the string contains a Chinese string
    :param s:
    :return:
    """
    for _char in s:
        if '\u4e00' <= _char <= '\u9fa5':
            return True
    return False


def merge(word_list):
    s = ''
    for w in word_list:
        s += w.split('/')[0]
    return s


def jaccard_similarity(s1, s2):
    if len(s1) == 0 or len(s2) == 0:
        return 0

    def add_space(s):
        return ' '.join(list(s))

    s1, s2 = add_space(s1), add_space(s2)
    cv = CountVectorizer(tokenizer=lambda s: s.split())
    corpus = [s1, s2]
    vectors = cv.fit_transform(corpus).toarray()
    numerator = np.sum(np.min(vectors, axis=0))
    denominator = np.sum(np.max(vectors, axis=0))
    return 1.0 * numerator / denominator
